@@ -408,3 +408,35 @@ int uwsgi_simple_wait_write_hook(int fd, int timeout) {
 
         return ret;
 }
+
+/*
+	simplified write to client
+	(generally used as fallback)
+*/
+int uwsgi_simple_write(struct wsgi_request *wsgi_req, char *buf, size_t len) {
+
+	wsgi_req->write_pos = 0;
+
+	for(;;) {
+                int ret = wsgi_req->socket->proto_write(wsgi_req, buf, len);
+                if (ret < 0) {
+                        if (!uwsgi.ignore_write_errors) {
+                                uwsgi_error("uwsgi_simple_write()");
+                        }
+                        wsgi_req->write_errors++;
+                        return -1;
+                }
+                if (ret == UWSGI_OK) {
+                        break;
+                }
+                ret = uwsgi_wait_write_req(wsgi_req);
+                if (ret < 0) { wsgi_req->write_errors++; return -1;}
+                if (ret == 0) {
+                        uwsgi_log("uwsgi_simple_write() TIMEOUT !!!\n");
+                        wsgi_req->write_errors++;
+                        return -1;
+                }
+        }
+        return 0;
+}
+
