@@ -132,10 +132,16 @@ void linux_namespace_jail() {
 
 	if (strcmp(uwsgi.ns, "/")) {
 		ns_tmp_mountpoint = uwsgi_concat2(uwsgi.ns, "/.uwsgi_ns_tmp_mountpoint");
-		mkdir(ns_tmp_mountpoint, S_IRWXU);
+		if (mkdir(ns_tmp_mountpoint, S_IRWXU) < 0) {
+			uwsgi_error("mkdir() ns_tmp_mountpoint");
+			exit(1);
+		}
 
 		ns_tmp_mountpoint2 = uwsgi_concat2(ns_tmp_mountpoint, "/.uwsgi_ns_tmp_mountpoint");
-		mkdir(ns_tmp_mountpoint2, S_IRWXU);
+		if (mkdir(ns_tmp_mountpoint2, S_IRWXU) < 0) {
+			uwsgi_error("mkdir() ns_tmp_mountpoint2");
+			exit(1);
+                }
 
 		if (mount(uwsgi.ns, ns_tmp_mountpoint, "none", MS_BIND, NULL)) {
 			uwsgi_error("mount()");
@@ -201,10 +207,16 @@ void linux_namespace_jail() {
 
 		unmounted = 0;
 		procmounts = fopen("/proc/self/mounts", "r");
+		if (!procmounts)
+			break;
 		while (fgets(line, 1024, procmounts) != NULL) {
 			delim0 = strchr(line, ' ');
+			if (!delim0)
+				continue;
 			delim0++;
 			delim1 = strchr(delim0, ' ');
+			if (!delim1)
+				continue;
 			*delim1 = 0;
 			// and now check for keep-mounts
 			if (uwsgi_is_a_keep_mount(delim0)) continue;
