@@ -121,7 +121,6 @@ int http_add_uwsgi_header(struct corerouter_peer *peer, char *hh, uint16_t hhlen
 
 	else if (!uwsgi_strncmp("CONTENT_LENGTH", 14, hh, keylen)) {
 		hr->content_length = uwsgi_str_num(val, vallen);
-		hr->session.can_keepalive = 0;
 	}
 
 	// in the future we could support chunked requests...
@@ -183,19 +182,28 @@ int http_headers_parse(struct corerouter_peer *peer) {
 	peer->out_pos = 0;
 
 	struct uwsgi_buffer *out = peer->out;
+	int found = 0;
 
 	// REQUEST_METHOD 
 	while (ptr < watermark) {
 		if (*ptr == ' ') {
 			if (uwsgi_buffer_append_keyval(out, "REQUEST_METHOD", 14, base, ptr - base)) return -1;
 			ptr++;
+			found = 1;
 			break;
 		}
+		else if (*ptr == '\r' || *ptr == '\n') break;
 		ptr++;
 	}
 
+        // ensure we have a method
+        if (!found) {
+          return -1;
+        }
+
 	// REQUEST_URI / PATH_INFO / QUERY_STRING
 	base = ptr;
+	found = 0;
 	while (ptr < watermark) {
 		if (*ptr == '?' && !query_string) {
 			// PATH_INFO must be url-decoded !!!
@@ -221,13 +229,20 @@ int http_headers_parse(struct corerouter_peer *peer) {
 				if (uwsgi_buffer_append_keyval(out, "QUERY_STRING", 12, query_string, ptr - query_string)) return -1;
 			}
 			ptr++;
+			found = 1;
 			break;
 		}
 		ptr++;
 	}
 
+        // ensure we have a URI
+        if (!found) {
+          return -1;
+        }
+
 	// SERVER_PROTOCOL
 	base = ptr;
+	found = 0;
 	while (ptr < watermark) {
 		if (*ptr == '\r') {
 			if (ptr + 1 >= watermark)
@@ -239,11 +254,17 @@ int http_headers_parse(struct corerouter_peer *peer) {
 				hr->session.can_keepalive = 1;
 			}
 			ptr += 2;
+			found = 1;
 			break;
 		}
 		ptr++;
 	}
 
+        // ensure we have a protocol
+        if (!found) {
+          return -1;
+        }
+
 	// SCRIPT_NAME
 	if (uwsgi_buffer_append_keyval(out, "SCRIPT_NAME", 11, "", 0)) return -1;
 
@@ -459,6 +480,8 @@ ssize_t hr_instance_read(struct corerouter_peer *peer) {
 	struct http_session *hr = (struct http_session *) peer->session;
         ssize_t len = cr_read(peer, "hr_instance_read()");
         if (!len) {
+		// disable keepalive on unread body
+		if (hr->content_length) hr->session.can_keepalive = 0;
 		if (hr->session.can_keepalive) {
 			peer->session->main_peer->disabled = 0;
 			hr->rnrn = 0;
@@ -575,9 +598,16 @@ ssize_t http_parse(struct corerouter_peer *main_peer) {
 				if (main_peer->in->pos > hr->content_length) {
 					main_peer->in->pos = hr->content_length;
 					hr->content_length = 0;
+					// on pipeline attempt, disable keepalive
+					hr->session.can_keepalive = 0;
 				}		
 				else {
 					hr->content_length -= main_peer->in->pos;
+					if (hr->content_length == 0) {
+						main_peer->disabled = 1;
+                                		// stop reading from the client
+                                		if (uwsgi_cr_set_hooks(main_peer, NULL, NULL)) return -1;
+					}
 				}
 			}
 		}
@@ -649,10 +679,11 @@ ssize_t http_parse(struct corerouter_peer *main_peer) {
         		new_peer->out->buf[2] = (uint8_t) ((pktsize >> 8) & 0xff);
 
 			if (hr->remains > 0) {
-				hr->session.can_keepalive = 0;
 				if (hr->content_length < hr->remains) { 
 					hr->remains = hr->content_length;
 					hr->content_length = 0;
+					// we need to avoid problems with pipelined requests
+					hr->session.can_keepalive = 0;
 				}
 				else {
 					hr->content_length -= hr->remains;
@@ -660,7 +691,7 @@ ssize_t http_parse(struct corerouter_peer *main_peer) {
 				if (uwsgi_buffer_append(new_peer->out, main_peer->in->buf + hr->headers_size + 1, hr->remains)) return -1;
 			}
 
-			if (hr->session.can_keepalive) {
+			if (hr->session.can_keepalive && hr->content_length == 0) {
 				main_peer->disabled = 1;
 				// stop reading from the client
 				if (uwsgi_cr_set_hooks(main_peer, NULL, NULL)) return -1;
