@@ -28,8 +28,8 @@ void *uwsgi_request_subhandler_web3(struct wsgi_request *wsgi_req, struct uwsgi_
                 Py_DECREF(pydictvalue);
         }
 
-        if (wsgi_req->uh.modifier1 == UWSGI_MODIFIER_MANAGE_PATH_INFO) {
-                wsgi_req->uh.modifier1 = 0;
+        if (wsgi_req->uh->modifier1 == UWSGI_MODIFIER_MANAGE_PATH_INFO) {
+                wsgi_req->uh->modifier1 = 0;
                 pydictkey = PyDict_GetItemString(wsgi_req->async_environ, "SCRIPT_NAME");
                 if (pydictkey) {
                         if (PyString_Check(pydictkey)) {
@@ -44,23 +44,9 @@ void *uwsgi_request_subhandler_web3(struct wsgi_request *wsgi_req, struct uwsgi_
                 }
         }
 
-        // if async_post is mapped as a file, directly use it as wsgi.input
-        if (wsgi_req->async_post) {
-#ifdef PYTHREE
-                wsgi_req->async_input = PyFile_FromFd(fileno((FILE *)wsgi_req->async_post), "web3_input", "rb", 0, NULL, NULL, NULL, 0);
-#else
-                wsgi_req->async_input = PyFile_FromFile(wsgi_req->async_post, "web3_input", "r", NULL);
-#endif
-        }
-        else {
-                // create wsgi.input custom object
-                wsgi_req->async_input = (PyObject *) PyObject_New(uwsgi_Input, &uwsgi_InputType);
-                ((uwsgi_Input*)wsgi_req->async_input)->wsgi_req = wsgi_req;
-                ((uwsgi_Input*)wsgi_req->async_input)->pos = 0;
-                ((uwsgi_Input*)wsgi_req->async_input)->readline_pos = 0;
-                ((uwsgi_Input*)wsgi_req->async_input)->readline_max_size = 0;
-
-        }
+        // create wsgi.input custom object
+        wsgi_req->async_input = (PyObject *) PyObject_New(uwsgi_Input, &uwsgi_InputType);
+        ((uwsgi_Input*)wsgi_req->async_input)->wsgi_req = wsgi_req;
 
         PyDict_SetItemString(wsgi_req->async_environ, "web3.input", wsgi_req->async_input);
 
@@ -111,16 +97,6 @@ void *uwsgi_request_subhandler_web3(struct wsgi_request *wsgi_req, struct uwsgi_
                 PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.core", PyInt_FromLong(wsgi_req->async_id));
         }
 
-        // cache this ?
-        if (uwsgi.cluster_fd >= 0) {
-                zero = PyString_FromString(uwsgi.cluster);
-                PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.cluster", zero);
-                Py_DECREF(zero);
-                zero = PyString_FromString(uwsgi.hostname);
-                PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.cluster_node", zero);
-                Py_DECREF(zero);
-        }
-
         PyDict_SetItemString(wsgi_req->async_environ, "uwsgi.node", wi->uwsgi_node);
 
 
@@ -164,18 +140,13 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 				goto clear; 
 			} 
 
-			// send the headers if not already sent
-        		if (!wsgi_req->headers_sent && wsgi_req->headers_hvec > 0) {
-                		uwsgi_python_do_send_headers(wsgi_req);
-        		}
-
 			Py_DECREF(spit_args);
 
 			if (PyString_Check((PyObject *)wsgi_req->async_placeholder)) {
 				char *content = PyString_AsString(wsgi_req->async_placeholder);
 				size_t content_len = PyString_Size(wsgi_req->async_placeholder);
 				UWSGI_RELEASE_GIL
-                		wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, content, content_len);
+				uwsgi_response_write_body_do(wsgi_req, content, content_len);
 				UWSGI_GET_GIL
 				uwsgi_py_check_write_errors {
                         		uwsgi_py_write_exception(wsgi_req);
@@ -192,11 +163,9 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 			if (!wsgi_req->async_placeholder) {
 				goto clear;
 			}
-#ifdef UWSGI_ASYNC
 			if (uwsgi.async > 1) {
 				return UWSGI_AGAIN;
 			}
-#endif
 		}
 		else {
 			uwsgi_log("invalid Web3 response.\n"); 
@@ -209,7 +178,9 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 	pychunk = PyIter_Next(wsgi_req->async_placeholder);
 
 	if (!pychunk) {
-		if (PyErr_Occurred()) PyErr_Print();
+		if (PyErr_Occurred()) {
+			uwsgi_manage_exception(wsgi_req, uwsgi.catch_exceptions);
+		}
 		goto clear;
 	}
 
@@ -218,7 +189,7 @@ int uwsgi_response_subhandler_web3(struct wsgi_request *wsgi_req) {
 		char *content = PyString_AsString(pychunk);
 		size_t content_len = PyString_Size(pychunk);
 		UWSGI_RELEASE_GIL
-		wsgi_req->response_size += wsgi_req->socket->proto_write(wsgi_req, content, content_len);
+		uwsgi_response_write_body_do(wsgi_req, content, content_len);
 		UWSGI_GET_GIL
 		uwsgi_py_check_write_errors {
 			uwsgi_py_write_exception(wsgi_req);
