@@ -36,6 +36,7 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 	// append remaining body...
 	if (wsgi_req->proto_parser_remains > 0) {
 		if (uwsgi_buffer_append(ub, wsgi_req->proto_parser_remains_buf, wsgi_req->proto_parser_remains)) {
+			uwsgi_buffer_destroy(ub);
 			uwsgi_log("unable to generate http request for %s\n", addr);
                		return UWSGI_ROUTE_NEXT;
 		}
@@ -43,9 +44,18 @@ static int uwsgi_routing_func_http(struct wsgi_request *wsgi_req, struct uwsgi_r
 	}
 
 	// ok now if have offload threads, directly use them
-	if (wsgi_req->socket->can_offload) {
+	if (!wsgi_req->post_file && !ur->custom && wsgi_req->socket->can_offload) {
+		// append buffered body
+		if (uwsgi.post_buffering > 0 && wsgi_req->post_cl > 0) {
+			if (uwsgi_buffer_append(ub, wsgi_req->post_buffering_buf, wsgi_req->post_cl)) {
+				uwsgi_buffer_destroy(ub);
+				uwsgi_log("unable to generate http request for %s\n", addr);
+               			return UWSGI_ROUTE_NEXT;
+			}
+		}
         	if (!uwsgi_offload_request_net_do(wsgi_req, addr, ub)) {
                 	wsgi_req->via = UWSGI_VIA_OFFLOAD;
+			wsgi_req->status = 202;
 			return UWSGI_ROUTE_BREAK;
                 }
 	}
@@ -82,10 +92,16 @@ static int uwsgi_router_http(struct uwsgi_route *ur, char *args) {
 	return 0;
 }
 
+static int uwsgi_router_proxyhttp(struct uwsgi_route *ur, char *args) {
+	ur->custom = 1;
+	return uwsgi_router_http(ur, args);
+}
+
 
 static void router_http_register(void) {
 
 	uwsgi_register_router("http", uwsgi_router_http);
+	uwsgi_register_router("proxyhttp", uwsgi_router_proxyhttp);
 }
 
 struct uwsgi_plugin router_http_plugin = {
