@@ -150,6 +150,7 @@ parse:
 
 
 ssize_t uwsgi_proto_fastcgi_read_body(struct wsgi_request *wsgi_req, char *buf, size_t len) {
+	int has_read = 0;
 	if (wsgi_req->proto_parser_remains > 0) {
                 size_t remains = UMIN(wsgi_req->proto_parser_remains, len);
                 memcpy(buf, wsgi_req->proto_parser_remains_buf, remains);
@@ -204,12 +205,15 @@ ssize_t uwsgi_proto_fastcgi_read_body(struct wsgi_request *wsgi_req, char *buf,
                                 wsgi_req->proto_parser_buf = tmp_buf;
                                 wsgi_req->proto_parser_buf_size += fcgi_all_len - (wsgi_req->proto_parser_buf_size - wsgi_req->proto_parser_pos);
                         }
-			goto gather;
+			if (!has_read) goto gather;
+			errno = EAGAIN;
+			return -1;
                 }
                 else {
 gather:
 			rlen = read(wsgi_req->fd, wsgi_req->proto_parser_buf + wsgi_req->proto_parser_pos,  wsgi_req->proto_parser_buf_size - wsgi_req->proto_parser_pos);
 			if (rlen > 0) {
+				has_read = 1;
 				wsgi_req->proto_parser_pos += rlen;
 				continue;
 			}
